@@ -1,7 +1,7 @@
 package missingfunction
 
 func channels(s string) {
-	undefinedChannels(c()) //@suggestedfix("undefinedChannels", "quickfix")
+	undefinedChannels(c()) //@suggestedfix("undefinedChannels", "quickfix", "")
 }
 
 func c() (<-chan string, chan string) {
